import json
import zipfile
import io
import os
from smile_id_core.Utilities import validate_sec_params


class ApiVersion:
    BUILD_NUMBER = 0
    MAJOR_VERSION = 2
    MINOR_VERSION = 0


IMAGE_FILE_EXTENSIONS = (".png", ".jpg")


def generate_zip_file(
    partner_id,
    callback_url,
    upload_url,
    partner_params,
    image_params,
    id_info_params,
    sec_params,
):
    info_json = prepare_info_json(
        partner_id,
        callback_url,
        upload_url,
        partner_params,
        image_params,
        id_info_params,
        sec_params,
    )
    zip_buffer = io.BytesIO()
    with zipfile.ZipFile(zip_buffer, "a", zipfile.ZIP_DEFLATED, False) as zip_file:
        zip_file.writestr("info.json", data=json.dumps(info_json))
        for image in image_params:
            image_file_path = image["image"]
            if isinstance(image_file_path, str) and image_file_path.lower().endswith(
                IMAGE_FILE_EXTENSIONS
            ):
                # TODO: do we really silently skip a file if its extension is different?
                zip_file.write(image_file_path, os.path.basename(image_file_path))
    return zip_buffer.getvalue()


def prepare_info_json(
    partner_id,
    callback_url,
    upload_url,
    partner_params,
    image_params,
    id_info_params,
    sec_params,
):
    validate_sec_params(sec_params)
    return {
        "package_information": {
            "apiVersion": {
                "buildNumber": ApiVersion.BUILD_NUMBER,
                "majorVersion": ApiVersion.MAJOR_VERSION,
                "minorVersion": ApiVersion.MINOR_VERSION,
            },
            "language": "python",
        },
        "misc_information": {
            **sec_params,
            "retry": "false",
            "partner_params": partner_params,
            "file_name": "selfie.zip",
            "smile_client_id": partner_id,
            "callback_url": callback_url,
            "userData": {
                "isVerifiedProcess": False,
                "name": "",
                "fbUserID": "",
                "firstName": "Bill",
                "lastName": "",
                "gender": "",
                "email": "",
                "phone": "",
                "countryCode": "+",
                "countryName": "",
            },
        },
        "id_info": id_info_params,
        "images": prepare_image_payload(image_params),
        "server_information": upload_url,
    }


def prepare_image_payload(image_params):
    return [prepare_image_entry_dict(**image) for image in image_params]


def prepare_image_entry_dict(image, image_type_id, **_):
    if isinstance(image, bytes):
        image = image.decode("utf-8")
    elif image.lower().endswith(IMAGE_FILE_EXTENSIONS):
        return {
            "image_type_id": image_type_id,
            "image": "",
            "file_name": os.path.basename(image),
        }
    return {
        "image_type_id": image_type_id,
        "image": image,
        "file_name": "",
    }


def validate_images(images_params):
    if not images_params:
        raise ValueError("Please ensure that you send through image details")

    if not isinstance(images_params, list):
        raise ValueError("Please ensure that you send through image details as a list")

    for image in images_params:
        if isinstance(image["image"], str) and image["image"].lower().endswith(
            IMAGE_FILE_EXTENSIONS
        ):
            if not os.path.exists(image["image"]):
                raise FileNotFoundError(
                    "No such file or directory %s" % (image["image"])
                )



import base64
import time
import unittest
from datetime import datetime
from unittest.mock import patch
from uuid import uuid4

from Crypto.Cipher import PKCS1_v1_5
from Crypto.PublicKey import RSA
from smile_id_core import Signature, IdApi, ServerError


class TestIdApi(unittest.TestCase):
    def setUp(self):
        self.key = RSA.generate(2048)
        self.public_key = self.key.publickey().export_key()
        self.api_key = base64.b64encode(self.public_key).decode("UTF-8")
        self.partner_id = "001"
        self.id_api = IdApi("001", self.api_key, 0)
        self.__reset_params()
        self.signatureObj = Signature(self.partner_id, self.api_key)
        self.cipher = PKCS1_v1_5.new(self.key.exportKey())

    def __reset_params(self):
        self.partner_params = {
            "user_id": str(uuid4()),
            "job_id": str(uuid4()),
            "job_type": 5,
        }
        self.id_info_params = {
            "first_name": "FirstName",
            "middle_name": "LastName",
            "last_name": "MiddleName",
            "country": "NG",
            "id_type": "PASSPORT",
            "id_number": "A00000000",
            "dob": "1989-09-20",
            "phone_number": "",
            "entered": True,
        }
        self.options_params = {
            "return_job_status": True,
            "return_history": True,
            "return_images": True,
        }

    def test_instance(self):
        self.__reset_params()
        self.assertEqual(self.id_api.partner_id, "001")
        self.assertEqual(self.id_api.api_key, self.api_key)
        self.assertEqual(
            self.id_api.url,
            "https://testapi.smileidentity.com/v1",
        )

    def test_no_partner_params(self):
        self.__reset_params()
        with self.assertRaises(ValueError) as ve:
            response = self.id_api.submit_job(None, self.id_info_params)
        self.assertEqual(
            ve.exception.args[0], "Please ensure that you send through partner params"
        )

    def test_no_id_info_params(self):
        self.__reset_params()
        with self.assertRaises(ValueError) as ve:
            response = self.id_api.submit_job(self.partner_params, None)
        self.assertEqual(
            ve.exception.args[0], "Please ensure that you send through ID Information"
        )

    def test_invalid_job_type(self):
        self.__reset_params()
        self.partner_params["job_type"] = 1
        with self.assertRaises(ValueError) as ve:
            response = self.id_api.submit_job(
                self.partner_params, self.id_info_params, False
            )
        self.assertEqual(
            ve.exception.args[0],
            "Please ensure that you are setting your job_type to 5 to query ID Api",
        )

    def test_id_info_params(self):
        self.__reset_params()
        self.id_info_params["country"] = None
        with self.assertRaises(ValueError) as ve:
            response = self.id_api.submit_job(self.partner_params, self.id_info_params)
        self.assertEqual(ve.exception.args[0], "key country cannot be empty")

        self.__reset_params()
        self.id_info_params["id_type"] = None
        with self.assertRaises(ValueError) as ve:
            response = self.id_api.submit_job(self.partner_params, self.id_info_params)
        self.assertEqual(ve.exception.args[0], "key id_type cannot be empty")

        self.__reset_params()
        self.id_info_params["id_number"] = None
        with self.assertRaises(ValueError) as ve:
            response = self.id_api.submit_job(self.partner_params, self.id_info_params)
        self.assertEqual(ve.exception.args[0], "key id_number cannot be empty")

    def get_id_response(self, signature=False):
        if signature:
            sec_timestamp = self.signatureObj.generate_signature(
                timestamp=datetime.now().isoformat()
            )
        else:
            sec_timestamp = self.signatureObj.generate_sec_key(
                timestamp=int(time.time())
            )
        return {
            "JSONVersion": "1.0.0",
            "SmileJobID": "0000000324",
            "PartnerParams": {
                "job_id": "D7t4PtgWk9kl",
                "user_id": "fffafbdc-073f-44b1-81f5-588866124ae2",
                "job_type": 5,
            },
            "ResultType": "ID Verification",
            "ResultText": "ID Number Validated",
            "ResultCode": "1012",
            "IsFinalResult": "true",
            "Actions": {
                "Verify_ID_Number": "Verified",
                "Return_Personal_Info": "Returned",
            },
            "Country": "NG",
            "IDType": "PASSPORT",
            "IDNumber": "A00000000",
            "ExpirationDate": "2024-02-13",
            "FullName": "DOE LEO JOE",
            "DOB": "2000-09-20",
            "Photo": "/9j/4AAQSAAAAAD//gASSVJJP/bAEMABAMDAw/wCUUP/Z",
            "PhoneNumber": "Not Available",
            "Gender": "Male",
            "Address": "Not Available",
            "FullData": {},
            "Source": "ID API",
            "timestamp": sec_timestamp.get("timestamp"),
            "signature": sec_timestamp.get("signature")
            if signature
            else sec_timestamp.get("sec_key"),
        }

    @staticmethod
    def _get_smile_services_response():
        return {
            "id_types": {
                "NG": {
                    "NIN": ["country", "id_type", "id_number", "user_id", "job_id"],
                    "CAC": [
                        "country",
                        "id_type",
                        "id_number",
                        "user_id",
                        "company",
                        "job_id",
                    ],
                    "TIN": ["country", "id_type", "id_number", "user_id", "job_id"],
                    "VOTER_ID": [
                        "country",
                        "id_type",
                        "id_number",
                        "user_id",
                        "job_id",
                    ],
                    "BVN": ["country", "id_type", "id_number", "user_id", "job_id"],
                    "PHONE_NUMBER": [
                        "country",
                        "id_type",
                        "id_number",
                        "user_id",
                        "job_id",
                        "first_name",
                        "last_name",
                    ],
                    "DRIVERS_LICENSE": [
                        "country",
                        "id_type",
                        "id_number",
                        "user_id",
                        "job_id",
                        "first_name",
                        "last_name",
                        "dob",
                    ],
                    "PASSPORT": [
                        "country",
                        "id_type",
                        "id_number",
                        "user_id",
                        "job_id",
                        "first_name",
                        "last_name",
                        "dob",
                    ],
                },
            }
        }

    def test_error_return_data(self):
        self.__reset_params()
        with self.assertRaises(ServerError) as ve:
            with patch("requests.post") as mocked_post, patch(
                "requests.get"
            ) as mocked_get:
                mocked_post.return_value.status_code = 400
                mocked_post.return_value.ok = True
                mocked_post.return_value.text.return_value = {
                    "code": "2204",
                    "error": "unauthorized",
                }
                mocked_post.return_value.json.return_value = {
                    "code": "2204",
                    "error": "unauthorized",
                }

                mocked_get.return_value.status_code = 200
                mocked_get.return_value.ok = True
                mocked_get.return_value.text.return_value = (
                    TestIdApi._get_smile_services_response()
                )
                mocked_get.return_value.json.return_value = (
                    TestIdApi._get_smile_services_response()
                )

                response = self.id_api.submit_job(
                    self.partner_params, self.id_info_params
                )
        self.assertEqual(
            ve.exception.args[0],
            "Failed to post entity to https://testapi.smileidentity.com/v1/id_verification, status=400, response={'code': '2204', 'error': 'unauthorized'}",
        )

    def test_validate_return_data(self):
        self.__reset_params()
        timestamp = int(time.time())
        sec_timestamp = self.signatureObj.generate_sec_key(timestamp=timestamp)
        with patch("requests.post") as mocked_post:
            mocked_post.return_value.status_code = 200
            mocked_post.return_value.ok = True
            mocked_post.return_value.text.return_value = self.get_id_response()
            mocked_post.return_value.json.return_value = self.get_id_response()

            response = self.id_api.submit_job(
                self.partner_params, self.id_info_params, False
            )

            self.assertEqual(response.status_code, 200)
            self.assertIsNotNone(response.json())



import io
import os
import tempfile
import zipfile

import pytest

from smile_id_core.image_upload import (
    prepare_image_entry_dict,
    prepare_info_json,
    generate_zip_file,
    prepare_image_payload,
    validate_images,
)


def test_prepare_image_entry_dict():
    assert prepare_image_entry_dict("directory/file.jpg", 5) == {
        "image_type_id": 5,
        "image": "",
        "file_name": "file.jpg",
    }
    assert prepare_image_entry_dict("directory/file.txt", 5) == {
        "image_type_id": 5,
        "image": "directory/file.txt",
        "file_name": "",
    }


def test_prepare_info_json():
    image_params = [
        {
            "image": "directory/file.jpg",
            "image_type_id": 5,
        }
    ]

    info_json = prepare_info_json(
        partner_id="partner_id",
        callback_url="callback_url",
        upload_url="upload_url",
        partner_params="partner_params",
        image_params=image_params,
        id_info_params="id_info_params",
        sec_params={
            "sec_key": "sec_key",
            "timestamp": "timestamp",
        },
    )
    assert info_json["id_info"] == "id_info_params"
    assert info_json["images"] == prepare_image_payload(image_params)
    assert info_json["misc_information"]["smile_client_id"] == "partner_id"


@pytest.fixture()
def temp_image_file():
    with tempfile.NamedTemporaryFile(delete=False, suffix=".jpg") as temp_image_file:
        temp_image_file.write(b"test image data")

    yield temp_image_file.name

    os.remove(temp_image_file.name)


def test_generate_zip_file(temp_image_file):

    assert temp_image_file.endswith(".jpg")

    image_params = [
        {
            "image": temp_image_file,
            "image_type_id": 5,
        }
    ]

    zip_stream = generate_zip_file(
        partner_id="partner_id",
        callback_url="callback_url",
        upload_url="upload_url",
        partner_params="partner_params",
        image_params=image_params,
        id_info_params="id_info_params",
        sec_params={"sec_key": "sec_key", "timestamp": "timestamp"},
    )

    zf = zipfile.ZipFile(io.BytesIO(zip_stream))
    assert zf.namelist() == ["info.json", os.path.basename(temp_image_file)]


def test_validate_images__ok_file_exists(temp_image_file):
    image_params = [
        {
            "image": temp_image_file,
            "image_type_id": 5,
        }
    ]

    assert validate_images(image_params) is None


def test_validate_images__error_file_not_found():
    image_params = [
        {
            "image": "nonexistent/file.jpg",
            "image_type_id": 5,
        }
    ]

    with pytest.raises(FileNotFoundError):
        validate_images(image_params)



import json
from typing import Dict

from smile_id_core.Utilities import (
    Utilities,
    get_signature,
    validate_sec_params,
    sid_server_map,
)
from smile_id_core.ServerError import ServerError
import requests

__all__ = ["IdApi"]


class IdApi:
    def __init__(self, partner_id: str, api_key: str, sid_server: str or int):
        if not partner_id or not api_key:
            raise ValueError("partner_id or api_key cannot be null or empty")
        self.partner_id = partner_id
        self.api_key = api_key
        if sid_server in [0, 1]:
            self.url = sid_server_map[sid_server]
        else:
            self.url = sid_server

    def submit_job(
        self,
        partner_params: Dict,
        id_params: Dict,
        use_validation_api=True,
        options_params: Dict = None,
    ):
        if not options_params:
            options_params = {}

        Utilities.validate_partner_params(partner_params)

        if not id_params:
            raise ValueError("Please ensure that you send through ID Information")

        Utilities.validate_id_params(
            self.url, id_params, partner_params, use_validation_api
        )

        if partner_params.get("job_type") != 5:
            raise ValueError(
                "Please ensure that you are setting your job_type to 5 to query ID Api"
            )

        sec_key_object = get_signature(
            self.partner_id, self.api_key, options_params.get("signature")
        )
        payload = self.__configure_json(partner_params, id_params, sec_key_object)
        response = self.__execute_http(payload)
        if response.status_code != 200:
            raise ServerError(
                "Failed to post entity to {}, status={}, response={}".format(
                    self.url + "/id_verification", response.status_code, response.json()
                )
            )
        return response

    def __configure_json(self, partner_params, id_params, sec_key):
        validate_sec_params(sec_key)
        payload = {
            **sec_key,
            "partner_id": self.partner_id,
            "partner_params": partner_params,
        }
        payload.update(id_params)
        return payload

    def __execute_http(self, payload):
        data = json.dumps(payload)
        resp = requests.post(
            url=self.url + "/id_verification",
            data=data,
            headers={
                "Accept": "application/json",
                "Accept-Language": "en_US",
                "Content-type": "application/json",
            },
        )
        return resp



import json
import time
from typing import Dict

import requests

from smile_id_core.image_upload import generate_zip_file, validate_images
from smile_id_core.IdApi import IdApi
from smile_id_core.Signature import Signature
from smile_id_core.Utilities import (
    Utilities,
    get_signature,
    validate_sec_params,
    sid_server_map,
)
from smile_id_core.ServerError import ServerError

__all__ = ["WebApi"]


class WebApi:
    def __init__(self, partner_id, call_back_url, api_key, sid_server):
        if not partner_id or not api_key:
            raise ValueError("partner_id or api_key cannot be null or empty")
        self.partner_id = partner_id
        self.call_back_url = call_back_url
        self.api_key = api_key
        self.sid_server = sid_server
        self.utilities = None

        if sid_server in [0, 1]:
            self.url = sid_server_map[sid_server]
        else:
            self.url = sid_server

    def submit_job(
        self,
        partner_params,
        images_params,
        id_info_params,
        options_params,
        use_validation_api=True,
    ):

        Utilities.validate_partner_params(partner_params)
        job_type = partner_params.get("job_type")

        if not id_info_params:
            if job_type == 5:
                Utilities.validate_id_params(
                    self.url, id_info_params, partner_params, use_validation_api
                )
            id_info_params = {
                "first_name": None,
                "middle_name": None,
                "last_name": None,
                "country": None,
                "id_type": None,
                "id_number": None,
                "dob": None,
                "phone_number": None,
                "entered": False,
            }

        if not options_params:
            options_params = {
                "return_job_status": True,
                "return_history": False,
                "return_images": False,
            }

        if job_type == 5:
            return self.__call_id_api(
                partner_params, id_info_params, use_validation_api, options_params
            )

        self.__validate_options(options_params)
        validate_images(images_params)
        Utilities.validate_id_params(
            self.url, id_info_params, partner_params, use_validation_api
        )
        self.__validate_return_data(options_params)

        sec_params = self._get_security_key_params(options_params)

        prep_upload = WebApi.execute_http(
            self.url + "/upload",
            self.__prepare_prep_upload_payload(partner_params, sec_params),
        )
        if prep_upload.status_code != 200:
            raise ServerError(
                "Failed to post entity to {}, status={}, response={}".format(
                    self.url + "/upload", prep_upload.status_code, prep_upload.json()
                )
            )
        else:
            prep_upload_json_resp = prep_upload.json()
            upload_url = prep_upload_json_resp["upload_url"]
            smile_job_id = prep_upload_json_resp["smile_job_id"]
            zip_stream = generate_zip_file(
                partner_id=self.partner_id,
                callback_url=self.call_back_url,
                image_params=images_params,
                partner_params=partner_params,
                id_info_params=id_info_params,
                upload_url=upload_url,
                sec_params=sec_params,
            )
            upload_response = WebApi.upload(upload_url, zip_stream)
            if upload_response.status_code != 200:
                raise ServerError(
                    "Failed to post entity to {}, status={}, response={}".format(
                        upload_url, upload_response.status_code, upload_response.json()
                    )
                )

            if options_params["return_job_status"]:
                self.utilities = Utilities(
                    self.partner_id, self.api_key, self.sid_server
                )
                job_status = self.poll_job_status(
                    0, partner_params, options_params, sec_params
                )
                return job_status
            else:
                return {"success": True, "smile_job_id": smile_job_id}

    def _get_security_key_params(self, options_params):
        return get_signature(
            self.partner_id, self.api_key, options_params.get("signature")
        )

    def __call_id_api(
        self,
        partner_params: Dict,
        id_info_params: Dict,
        use_validation_api: bool,
        options_params: Dict,
    ):
        id_api = IdApi(self.partner_id, self.api_key, self.sid_server)
        return id_api.submit_job(
            partner_params, id_info_params, use_validation_api, options_params
        )

    def __validate_options(self, options_params):
        if not self.call_back_url and not options_params:
            raise ValueError(
                "Please choose to either get your response via the callback or job status query"
            )

        if options_params:
            for key in options_params:
                if key != "optional_callback" and not type(options_params[key]) == bool:
                    raise ValueError(key + " needs to be a boolean")

    def __validate_return_data(self, options):
        if not self.call_back_url and not options["return_job_status"]:
            raise ValueError(
                "Please choose to either get your response via the callback or job status query"
            )

    def __get_sec_key(self):
        sec_key_gen = Signature(self.partner_id, self.api_key)
        return sec_key_gen.generate_sec_key()

    def __prepare_prep_upload_payload(self, partner_params: Dict, sec_params: Dict):
        validate_sec_params(sec_params)

        return {
            "file_name": "selfie.zip",
            "smile_client_id": self.partner_id,
            "partner_params": partner_params,
            "model_parameters": {},
            "callback_url": self.call_back_url,
            **sec_params,
        }

    def poll_job_status(
        self, counter, partner_params, options_params, sec_params: Dict
    ):
        if sec_params is None:
            sec_params = self._get_security_key_params(options_params)

        validate_sec_params(sec_params)
        counter = counter + 1
        if counter < 4:
            time.sleep(2)
        else:
            time.sleep(4)

        job_status = self.utilities.get_job_status(
            partner_params, options_params, sec_params
        )
        job_status_response = job_status.json()
        if not job_status_response["job_complete"] and counter < 20:
            return self.poll_job_status(
                counter, partner_params, options_params, sec_params
            )

        return job_status

    @staticmethod
    def execute_http(url, payload):
        data = json.dumps(payload)
        resp = requests.post(
            url=url,
            data=data,
            headers={
                "Accept": "application/json",
                "Accept-Language": "en_US",
                "Content-type": "application/json",
            },
        )
        return resp

    @staticmethod
    def upload(url, file):
        resp = requests.put(
            url=url, data=file, headers={"Content-type": "application/zip"}
        )
        return resp



from smile_id_core.Utilities import Utilities
from smile_id_core.IdApi import IdApi
from smile_id_core.WebApi import WebApi
from smile_id_core.Signature import Signature
from smile_id_core.ServerError import ServerError

__all__ = ["IdApi", "Signature", "Utilities", "WebApi", "ServerError"]



import base64
import hashlib
import hmac
import time
import unittest
from datetime import datetime

from Crypto.Cipher import PKCS1_v1_5
from Crypto.PublicKey import RSA

from smile_id_core import Signature


class TestSignature(unittest.TestCase):
    def setUp(self):
        self.key = RSA.generate(2048)
        self.public_key = self.key.publickey().export_key()
        self.api_key = base64.b64encode(self.public_key).decode("UTF-8")
        self.partner_id = "001"
        self.signatureObj = Signature(self.partner_id, self.api_key)
        self.cipher = PKCS1_v1_5.new(self.key.exportKey())

    def test_partner_id_api_key(self):
        self.assertEqual(self.signatureObj.partner_id, self.partner_id)
        # self.assertEqual(self.public_key, self.signatureObj.decoded_api_key)

    def test_generate_sec_key(self):
        timestamp = int(time.time())
        sec_timestamp = self.signatureObj.generate_sec_key(timestamp=timestamp)
        self.assertEqual(sec_timestamp["timestamp"], timestamp)

        hashed = hashlib.sha256(
            "{}:{}".format(int(self.partner_id), timestamp).encode("utf-8")
        ).hexdigest()
        encrypted, hashed2 = sec_timestamp["sec_key"].split("|")
        self.assertEqual(hashed, hashed2)

    def test_generate_signature(self):
        timestamp = datetime.now().isoformat()
        signature = self.signatureObj.generate_signature(timestamp=timestamp)
        self.assertEqual(signature["timestamp"], timestamp)

        hmac_new = hmac.new(self.api_key.encode(), digestmod=hashlib.sha256)
        hmac_new.update(timestamp.encode("utf-8"))
        hmac_new.update(str(self.partner_id).encode("utf-8"))
        hmac_new.update("sid_request".encode("utf-8"))
        calculated_signature = base64.b64encode(hmac_new.digest()).decode("utf-8")

        self.assertEqual(signature["signature"], calculated_signature)

    # TODO: Confirm sec key tests
    def test_confirm_sec_key(self):
        pass



import json
import unittest
from typing import Dict

import responses


class TestCaseWithStubs(
    unittest.TestCase,
):
    def stub_get_job_status(self, sec_key, job_complete=True, with_error=None):
        if with_error:
            job_status_response = {
                "status": 400,
                "json": {"code": "2098", "error": with_error},
            }
        else:
            job_status_response = {
                "status": 200,
                "json": self._get_job_status_response(sec_key, job_complete),
            }

        responses.add(
            responses.POST,
            "https://testapi.smileidentity.com/v1/job_status",
            **job_status_response,
        )
        return job_status_response["json"]

    def _get_job_status_response(self, sec_key, job_complete=True):
        return {
            "signature": sec_key.get("signature") or sec_key.get("sec_key"),
            "timestamp": sec_key.get("timestamp"),
            "job_complete": job_complete,
            "job_success": True,
            "result": {
                "ResultText": "Enroll User",
                "ResultType": "SAIA",
                "SmileJobID": "0000001897",
                "JSONVersion": "1.0.0",
                "IsFinalResult": "true",
                "PartnerParams": {
                    "job_id": "52d0de86-be3b-4219-9e96-8195b0018944",
                    "user_id": "e54e0e98-8b8c-4215-89f5-7f9ea42bf650",
                    "job_type": 4,
                },
                "ConfidenceValue": "100",
                "IsMachineResult": "true",
            },
            "image_links": {
                "selfie_image": "https://smile-fr-results.s3.us-west-2.amazonaws.com/test/000000/023/023-0000001897-LoRSpxJUzmYgYS2R00XpaHJYLOiNXN/SID_Preview_FULL.jpg"
            },
            "code": "2302",
            "history": [
                {
                    "ResultCode": "1210",
                    "ResultText": "Enroll User",
                    "ResultType": "DIVA",
                    "SmileJobID": "0000000857",
                    "JSONVersion": "1.0.0",
                    "IsFinalResult": "true",
                    "PartnerParams": {
                        "job_id": "52d0de86-be3b-4219-9e96-8195b0018944",
                        "user_id": "1511bf02-801a-4b57-ac8e-ef17e26bfeb4",
                        "job_type": "1",
                        "optional_info": "Partner can put whatever they want as long as it is a string",
                        "more_optional_info": "There can be as much or as little or no optional info",
                    },
                },
                {
                    "ResultCode": "0814",
                    "ResultText": "Provisional Enroll - Under Review",
                    "SmileJobID": "0000000857",
                    "ConfidenceValue": "97.000000",
                    "PartnerParams": {
                        "job_id": "52d0de86-be3b-4219-9e96-8195b0018944",
                        "user_id": "1511bf02-801a-4b57-ac8e-ef17e26bfeb4",
                        "job_type": "1",
                        "optional_info": "Partner can put whatever they want as long as it is a string",
                        "more_optional_info": "There can be as much or as little or no optional info",
                    },
                },
                {
                    "DOB": "1990-01-01",
                    "IDType": "BVN",
                    "Country": "Nigeria",
                    "FullName": "Peter Parker",
                    "ExpirationDate": "Not Available",
                    "IDNumber": "A01234567",
                    "ResultCode": "1012",
                    "ResultText": "ID Validated",
                    "ResultType": "ID Verification",
                    "SmileJobID": "0000000857",
                    "PartnerParams": {
                        "job_id": "52d0de86-be3b-4219-9e96-8195b0018944",
                        "user_id": "1511bf02-801a-4b57-ac8e-ef17e26bfeb4",
                        "job_type": "1",
                        "optional_info": "Partner can put whatever they want as long as it is a string",
                        "more_optional_info": "There can be as much or as little or no optional info",
                        "ExpirationDate": "Not Available",
                    },
                },
            ],
        }

    def _get_pre_upload_response(self, sec_key: Dict):
        return {
            "signature": sec_key.get("signature") or sec_key.get("sec_key"),
            "timestamp": sec_key.get("timestamp"),
            "upload_url": "https://some_url.com",
            "smile_job_id": "0000000857",
            "job_complete": True,
            "job_success": True,
            "result": {
                "ResultText": "Enroll User",
                "ResultType": "SAIA",
                "SmileJobID": "0000001897",
                "JSONVersion": "1.0.0",
                "IsFinalResult": "true",
                "PartnerParams": {
                    "job_id": "52d0de86-be3b-4219-9e96-8195b0018944",
                    "user_id": "e54e0e98-8b8c-4215-89f5-7f9ea42bf650",
                    "job_type": 4,
                },
                "ConfidenceValue": "100",
                "IsMachineResult": "true",
            },
            "image_links": {
                "selfie_image": "https://smile-fr-results.s3.us-west-2.amazonaws.com/test/000000/023/023-0000001897-LoRSpxJUzmYgYS2R00XpaHJYLOiNXN/SID_Preview_FULL.jpg"
            },
            "code": "2302",
            "history": [
                {
                    "ResultCode": "1210",
                    "ResultText": "Enroll User",
                    "ResultType": "DIVA",
                    "SmileJobID": "0000000857",
                    "JSONVersion": "1.0.0",
                    "IsFinalResult": "true",
                    "PartnerParams": {
                        "job_id": "52d0de86-be3b-4219-9e96-8195b0018944",
                        "user_id": "1511bf02-801a-4b57-ac8e-ef17e26bfeb4",
                        "job_type": "1",
                        "optional_info": "Partner can put whatever they want as long as it is a string",
                        "more_optional_info": "There can be as much or as little or no optional info",
                    },
                },
                {
                    "ResultCode": "0814",
                    "ResultText": "Provisional Enroll - Under Review",
                    "SmileJobID": "0000000857",
                    "ConfidenceValue": "97.000000",
                    "PartnerParams": {
                        "job_id": "52d0de86-be3b-4219-9e96-8195b0018944",
                        "user_id": "1511bf02-801a-4b57-ac8e-ef17e26bfeb4",
                        "job_type": "1",
                        "optional_info": "Partner can put whatever they want as long as it is a string",
                        "more_optional_info": "There can be as much or as little or no optional info",
                    },
                },
                {
                    "DOB": "1990-01-01",
                    "IDType": "BVN",
                    "Country": "Nigeria",
                    "FullName": "Peter Parker",
                    "ExpirationDate": "Not Available",
                    "IDNumber": "A01234567",
                    "ResultCode": "1012",
                    "ResultText": "ID Validated",
                    "ResultType": "ID Verification",
                    "SmileJobID": "0000000857",
                    "PartnerParams": {
                        "job_id": "52d0de86-be3b-4219-9e96-8195b0018944",
                        "user_id": "1511bf02-801a-4b57-ac8e-ef17e26bfeb4",
                        "job_type": "1",
                        "optional_info": "Partner can put whatever they want as long as it is a string",
                        "more_optional_info": "There can be as much or as little or no optional info",
                        "ExpirationDate": "Not Available",
                    },
                },
            ],
        }

    def stub_upload_request(self, sec_key, fail_with_message=None):
        post_response = self._get_pre_upload_response(sec_key)
        responses.add(
            responses.POST,
            "https://testapi.smileidentity.com/v1/upload",
            status=200,
            json=post_response,
        )
        upload_response = {"status": 200, "json": {}}
        if fail_with_message:
            upload_response = {
                "status": 400,
                "json": {"code": "2205", "error": fail_with_message},
            }
        responses.add(responses.PUT, post_response["upload_url"], **upload_response)
        return post_response

    def assert_request_called_with(self, url, method, body):
        called_requests = [
            call.request
            for call in responses.calls
            if call.request.url == url and call.request.method == method
        ]
        size = len(called_requests)
        if size == 0:
            self.fail(f"{method} {url} not called")
        # elif size == 1:
        #     self.assertEqual(called_requests[0].body, json.dumps(body))
        else:
            found = False
            for request in called_requests:
                try:
                    self.assertEqual(request.body, json.dumps(body))
                    found = True
                except AssertionError as e:
                    pass
            if not found:
                self.fail(f"No request match call with body {body}")



from setuptools import setup, find_packages
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="smile_id_core",
    version="1.0.0",
    description="The official Smile Identity package exposes four classes namely; the WebApi class, the IDApi class, the Signature class and the Utilities class.",
    packages=find_packages(exclude=["*.tests", "*.tests.*"]),
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    url="https://github.com/smileidentity/smile-identity-core-python",
    author="Smile Identity",
    author_email="support@smileidentity.com",
    install_requires=[
        "requests~=2.24.0",
        "pycryptodome~=3.9.8",
    ],
)



import base64
import hashlib
import hmac
import time
from datetime import datetime

from Crypto.Cipher import PKCS1_v1_5
from Crypto.PublicKey import RSA

__all__ = ["Signature"]


class Signature:
    def __init__(self, partner_id: str, api_key: str):
        if not partner_id or not api_key:
            raise ValueError("partner_id or api_key cannot be null or empty")
        self.partner_id = partner_id
        self.api_key = api_key

    def generate_sec_key(self, timestamp=None):
        public_key = RSA.importKey(base64.b64decode(self.api_key))
        cipher = PKCS1_v1_5.new(public_key)
        if timestamp is None:
            timestamp = int(time.time())
        hashed = self.__get_hash(timestamp)
        encrypted = base64.b64encode(cipher.encrypt(hashed.encode("utf-8")))

        signature = "{}|{}".format(encrypted.decode(encoding="UTF-8"), hashed)
        return {"sec_key": signature, "timestamp": timestamp}

    def __get_hash(self, timestamp):
        to_hash = "{}:{}".format(int(self.partner_id), timestamp)
        new_hash = str(to_hash).encode("utf-8")
        return hashlib.sha256(new_hash).hexdigest()

    def generate_signature(self, timestamp=None):
        _timestamp = timestamp
        if _timestamp is None:
            _timestamp = datetime.now().isoformat()
        hmac_new = hmac.new(self.api_key.encode("utf-8"), digestmod=hashlib.sha256)
        hmac_new.update(_timestamp.encode("utf-8"))
        hmac_new.update(str(self.partner_id).encode("utf-8"))
        hmac_new.update("sid_request".encode("utf-8"))
        calculated_signature = base64.b64encode(hmac_new.digest())
        return {
            "signature": calculated_signature.decode("utf-8"),
            "timestamp": _timestamp,
        }

    def confirm_signature(self, timestamp, msg_signature):
        return self.generate_signature(timestamp)["signature"] == msg_signature

    def confirm_sec_key(self, timestamp, sec_key):
        encrypted, hashed = sec_key.split("|")
        local_hash = self.__get_hash(timestamp)
        # python libraries only allow decryption from a private key
        # TODO: re look at this
        return True



import json
from typing import Dict

import requests

from smile_id_core.Signature import Signature
from smile_id_core.ServerError import ServerError

__all__ = ["Utilities", "get_signature", "validate_sec_params", "sid_server_map"]

sid_server_map = {
    0: "https://testapi.smileidentity.com/v1",
    1: "https://api.smileidentity.com/v1",
}


class Utilities:
    def __init__(self, partner_id, api_key, sid_server):
        if not partner_id or not api_key:
            raise ValueError("partner_id or api_key cannot be null or empty")
        self.partner_id = partner_id
        self.api_key = api_key
        self.sid_server = sid_server
        if sid_server in [0, 1]:
            self.url = sid_server_map[sid_server]
        else:
            self.url = sid_server

    def get_job_status(self, partner_params, option_params, sec_params):
        if sec_params is None:
            sec_params = get_signature(
                self.partner_id, self.api_key, option_params.get("signature")
            )

        validate_sec_params(sec_params)
        Utilities.validate_partner_params(partner_params)
        if not option_params or option_params is None:
            options = {
                "return_job_status": True,
                "return_history": False,
                "return_images": False,
            }
        else:
            options = option_params
        return self.__query_job_status(
            partner_params.get("user_id"),
            partner_params.get("job_id"),
            options,
            sec_params,
        )

    def __query_job_status(self, user_id, job_id, option_params, sec_params):
        job_status = Utilities.execute_post(
            self.url + "/job_status",
            self.__configure_job_query(user_id, job_id, option_params, sec_params),
        )
        if job_status.status_code != 200:
            raise ServerError(
                "Failed to post entity to {}, response={}:{} - {}".format(
                    self.url + "/job_status",
                    job_status.status_code,
                    job_status.reason,
                    job_status.json(),
                )
            )
        else:
            job_status_json_resp = job_status.json()
            timestamp = job_status_json_resp["timestamp"]
            server_signature = job_status_json_resp["signature"]
            signature = Signature(self.partner_id, self.api_key)
            if option_params.get("signature"):
                valid = signature.confirm_signature(timestamp, server_signature)
            else:
                valid = signature.confirm_sec_key(timestamp, server_signature)
            if not valid:
                raise ServerError(
                    "Unable to confirm validity of the job_status response"
                )
            return job_status

    def __configure_job_query(self, user_id, job_id, options, sec_params):
        return {
            **sec_params,
            "partner_id": self.partner_id,
            "job_id": job_id,
            "user_id": user_id,
            "image_links": options["return_images"],
            "history": options["return_history"],
        }

    @staticmethod
    def validate_partner_params(partner_params):
        if not partner_params:
            raise ValueError("Please ensure that you send through partner params")

        if (
            not partner_params.get("user_id")
            or not partner_params.get("job_id")
            or not partner_params.get("job_type")
        ):
            raise ValueError("Partner Parameter Arguments may not be null or empty")

        if not isinstance(partner_params.get("user_id"), str):
            raise ValueError("Please ensure user_id is a string")

        if not isinstance(partner_params.get("job_id"), str):
            raise ValueError("Please ensure job_id is a string")

        if not isinstance(partner_params.get("job_type"), int):
            raise ValueError("Please ensure job_id is a number")

    @staticmethod
    def validate_id_params(
        sid_server, id_info_params, partner_params, use_validation_api=True
    ):
        if not id_info_params.get("entered"):
            return

        for field in ["country", "id_type", "id_number"]:
            if field in id_info_params:
                if id_info_params[field]:
                    continue
                else:
                    raise ValueError("key " + field + " cannot be empty")
            else:
                raise ValueError("key " + field + " cannot be empty")
        if not use_validation_api:
            return

        response = Utilities.get_smile_id_services(sid_server)
        if response.status_code != 200:
            raise ServerError(
                "Failed to get to {}, status={}, response={}".format(
                    "/services", response.status_code, response.json()
                )
            )
        response_json = response.json()
        if response_json["id_types"]:
            if not id_info_params["country"] in response_json["id_types"]:
                raise ValueError("country " + id_info_params["country"] + " is invalid")
            selected_country = response_json["id_types"][id_info_params["country"]]
            if not id_info_params["id_type"] in selected_country:
                raise ValueError("id_type " + id_info_params["id_type"] + " is invalid")
            id_params = selected_country[id_info_params["id_type"]]
            for key in id_params:
                if key not in id_info_params and key not in partner_params:
                    raise ValueError("key " + key + " is required")
                if key in id_info_params and not id_info_params[key]:
                    raise ValueError("key " + key + " cannot be empty")
                if key in partner_params and not partner_params[key]:
                    raise ValueError("key " + key + " cannot be empty")

    @staticmethod
    def get_smile_id_services(sid_server):
        if sid_server in [0, 1]:
            url = sid_server_map[sid_server]
        else:
            url = sid_server
        response = Utilities.execute_get(url + "/services")
        if response.status_code != 200:
            raise ServerError(
                "Failed to get to {}, status={}, response={}".format(
                    url + "/services", response.status_code, response.json()
                )
            )
        return response

    @staticmethod
    def execute_get(url):
        resp = requests.get(
            url=url,
            headers={
                "Accept": "application/json",
                "Accept-Language": "en_US",
            },
        )
        return resp

    @staticmethod
    def execute_post(url, payload):
        data = json.dumps(payload)
        resp = requests.post(
            url=url,
            data=data,
            headers={
                "Accept": "application/json",
                "Accept-Language": "en_US",
                "Content-type": "application/json",
            },
        )
        return resp


def validate_sec_params(sec_key_dict: Dict):
    if not sec_key_dict.get("sec_key") and not sec_key_dict.get("signature"):
        raise Exception("Missing key, must provide a 'sec_key' or 'signature' field")
    if not sec_key_dict.get("timestamp"):
        raise Exception("Missing 'timestamp' field")


def get_signature(partner_id, api_key, is_signature):
    sec_key_gen = Signature(partner_id, api_key)
    sec_key_object = (
        sec_key_gen.generate_signature()
        if is_signature
        else sec_key_gen.generate_sec_key()
    )
    sec_key = sec_key_object.get("sec_key")
    signature = sec_key_object.get("signature")
    payload = {"timestamp": sec_key_object["timestamp"]}
    if sec_key:
        payload.update({"sec_key": sec_key})
    else:
        payload.update({"signature": signature})
    return payload



import base64
import time
from datetime import datetime
from typing import Dict
from uuid import uuid4

import responses
from Crypto.Cipher import PKCS1_v1_5
from Crypto.PublicKey import RSA

from smile_id_core import WebApi, Signature, ServerError
from tests.stub_mixin import TestCaseWithStubs


class TestWebApi(TestCaseWithStubs):
    def setUp(self):
        self.key = RSA.generate(2048)
        self.public_key = self.key.publickey().export_key()
        self.api_key = base64.b64encode(self.public_key).decode("UTF-8")
        self.partner_id = "001"
        self.__reset_params()
        self.web_api = WebApi("001", "https://a_callback.com", self.api_key, 0)
        self.signatureObj = Signature(self.partner_id, self.api_key)
        self.cipher = PKCS1_v1_5.new(self.key.exportKey())

    def __reset_params(self):
        self.partner_params = {
            "user_id": str(uuid4()),
            "job_id": str(uuid4()),
            "job_type": 1,
        }
        self.id_info_params = {
            "first_name": "FirstName",
            "middle_name": "LastName",
            "last_name": "MiddleName",
            "country": "NG",
            "id_type": "PASSPORT",
            "id_number": "A00000000",
            "dob": "1989-09-20",
            "phone_number": "",
            "entered": True,
        }
        self.image_params = []
        self.image_params.append({"image_type_id": "2", "image": "base6image"})
        self.options_params = {
            "return_job_status": True,
            "return_history": True,
            "return_images": True,
        }

    def test_instance(self):
        self.__reset_params()
        self.assertEqual(self.web_api.partner_id, "001")
        self.assertEqual(self.web_api.api_key, self.api_key)
        self.assertEqual(self.web_api.call_back_url, "https://a_callback.com")
        self.assertEqual(
            self.web_api.url,
            "https://testapi.smileidentity.com/v1",
        )

    def test_no_image_params(self):
        self.__reset_params()
        with self.assertRaises(ValueError) as ve:
            response = self.web_api.submit_job(
                self.partner_params, None, self.id_info_params, self.options_params
            )
        self.assertEqual(
            ve.exception.args[0], "Please ensure that you send through image details"
        )

    def test_no_partner_params(self):
        self.__reset_params()
        with self.assertRaises(ValueError) as ve:
            response = self.web_api.submit_job(
                None, self.image_params, self.id_info_params, self.options_params
            )
        self.assertEqual(
            ve.exception.args[0], "Please ensure that you send through partner params"
        )

    def test_missing_partner_params(self):
        self.__reset_params()
        self.partner_params["user_id"] = None
        with self.assertRaises(ValueError) as ve:
            response = self.web_api.submit_job(
                self.partner_params,
                self.image_params,
                self.id_info_params,
                self.options_params,
            )
        value_exception = ve.exception
        self.assertEqual(
            value_exception.args[0],
            "Partner Parameter Arguments may not be null or empty",
        )

        self.__reset_params()
        self.partner_params["job_id"] = None
        with self.assertRaises(ValueError) as ve:
            response = self.web_api.submit_job(
                self.partner_params,
                self.image_params,
                self.id_info_params,
                self.options_params,
            )
        self.assertEqual(
            ve.exception.args[0],
            "Partner Parameter Arguments may not be null or empty",
        )

        self.__reset_params()
        self.partner_params["job_type"] = None
        with self.assertRaises(ValueError) as ve:
            response = self.web_api.submit_job(
                self.partner_params,
                self.image_params,
                self.id_info_params,
                self.options_params,
            )
        self.assertEqual(
            ve.exception.args[0],
            "Partner Parameter Arguments may not be null or empty",
        )

    def test_id_info_params(self):
        self.__reset_params()
        self.id_info_params["country"] = None
        with self.assertRaises(ValueError) as ve:
            response = self.web_api.submit_job(
                self.partner_params,
                self.image_params,
                self.id_info_params,
                self.options_params,
            )
        self.assertEqual(ve.exception.args[0], "key country cannot be empty")

        self.__reset_params()
        self.id_info_params["id_type"] = None
        with self.assertRaises(ValueError) as ve:
            response = self.web_api.submit_job(
                self.partner_params,
                self.image_params,
                self.id_info_params,
                self.options_params,
            )
        self.assertEqual(ve.exception.args[0], "key id_type cannot be empty")

        self.__reset_params()
        self.id_info_params["id_number"] = None
        with self.assertRaises(ValueError) as ve:
            response = self.web_api.submit_job(
                self.partner_params,
                self.image_params,
                self.id_info_params,
                self.options_params,
            )
        self.assertEqual(ve.exception.args[0], "key id_number cannot be empty")

    def test_non_valid_image(self):
        self.__reset_params()
        self.image_params.append({"image_type_id": "0", "image": "path/to/image.jpg"})
        with self.assertRaises(FileNotFoundError) as ve:
            response = self.web_api.submit_job(
                self.partner_params,
                self.image_params,
                self.id_info_params,
                self.options_params,
            )
        self.assertEqual(
            ve.exception.args[0], "No such file or directory path/to/image.jpg"
        )

    def test_boolean_options_params_non_jt5(self):
        self.__reset_params()
        self.options_params["return_job_status"] = "Test"
        with self.assertRaises(ValueError) as ve:
            response = self.web_api.submit_job(
                self.partner_params,
                self.image_params,
                self.id_info_params,
                self.options_params,
                False,
            )
        self.assertEqual(
            ve.exception.args[0], "return_job_status needs to be a boolean"
        )

        self.__reset_params()
        self.options_params["return_history"] = "tEST"
        with self.assertRaises(ValueError) as ve:
            response = self.web_api.submit_job(
                self.partner_params,
                self.image_params,
                self.id_info_params,
                self.options_params,
                False,
            )
        self.assertEqual(ve.exception.args[0], "return_history needs to be a boolean")

        self.__reset_params()
        self.options_params["return_images"] = "tEST"
        with self.assertRaises(ValueError) as ve:
            response = self.web_api.submit_job(
                self.partner_params,
                self.image_params,
                self.id_info_params,
                self.options_params,
                False,
            )
        self.assertEqual(ve.exception.args[0], "return_images needs to be a boolean")

        self.__reset_params()
        self.options_params["signature"] = "tEST"
        with self.assertRaises(ValueError) as ve:
            response = self.web_api.submit_job(
                self.partner_params,
                self.image_params,
                self.id_info_params,
                self.options_params,
                False,
            )
        self.assertEqual(ve.exception.args[0], "signature needs to be a boolean")

    @responses.activate
    def test_submit_job_should_raise_error_when_pre_upload_fails(self):
        responses.add(
            responses.POST,
            "https://testapi.smileidentity.com/v1/upload",
            status=400,
            json={
                "code": "2204",
                "error": "unauthorized",
            },
        )
        self.__reset_params()
        with self.assertRaises(ServerError) as ve:
            self.web_api.submit_job(
                self.partner_params,
                self.image_params,
                self.id_info_params,
                self.options_params,
                False,
            )
        self.assertEqual(
            ve.exception.args[0],
            "Failed to post entity to https://testapi.smileidentity.com/v1/upload, status=400, response={'code': '2204', 'error': 'unauthorized'}",
        )

    @responses.activate
    def test_submit_job_should_raise_error_when_upload_fails(self):
        sec_key = self._get_sec_key(False)
        error = "Failed to upload zip"
        post_response = self.stub_upload_request(sec_key, error)

        self.__reset_params()
        with self.assertRaises(ServerError) as ve:
            self.web_api.submit_job(
                self.partner_params,
                self.image_params,
                self.id_info_params,
                self.options_params,
                False,
            )

        response = {"code": "2205", "error": error}
        self.assertEqual(
            ve.exception.args[0],
            f"Failed to post entity to {post_response['upload_url']}, status=400, response={response}",
        )

    @responses.activate
    def test_validate_return_data(self):
        self.__reset_params()
        sec_key = self._get_sec_key(False)
        post_response = self.stub_upload_request(sec_key)
        self.stub_get_job_status(sec_key, False)
        job_status_response = self.stub_get_job_status(sec_key, True)

        response = self.web_api.submit_job(
            self.partner_params,
            self.image_params,
            self.id_info_params,
            self.options_params,
            False,
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), job_status_response)

    def _get_sec_key(self, signature):
        if signature:
            sec_key = self.signatureObj.generate_signature(timestamp=datetime.now())
        else:
            sec_key = self.signatureObj.generate_sec_key(timestamp=int(time.time()))
        return sec_key



import base64
import time

from uuid import uuid4

import responses
from Crypto.Cipher import PKCS1_v1_5
from Crypto.PublicKey import RSA

from smile_id_core import Signature, Utilities
from tests.stub_mixin import TestCaseWithStubs


class TestUtilities(TestCaseWithStubs):
    def setUp(self):
        self.key = RSA.generate(2048)
        self.public_key = self.key.publickey().export_key()
        self.api_key = base64.b64encode(self.public_key).decode("UTF-8")
        self.partner_id = "001"
        self.signatureObj = Signature(self.partner_id, self.api_key)
        self.cipher = PKCS1_v1_5.new(self.key.exportKey())
        self.__reset_params()
        self.utilities = Utilities(self.partner_id, self.public_key, 0)

    def __reset_params(self):
        self.partner_params = {
            "user_id": str(uuid4()),
            "job_id": str(uuid4()),
            "job_type": 1,
        }
        self.id_info_params = {
            "first_name": "FirstName",
            "middle_name": "LastName",
            "last_name": "MiddleName",
            "country": "NG",
            "id_type": "PASSPORT",
            "id_number": "A00000000",
            "dob": "1989-09-20",
            "phone_number": "",
            "entered": True,
        }
        self.image_params = []
        self.image_params.append({"image_type_id": "2", "image": "base6image"})
        self.options_params = {
            "return_job_status": True,
            "return_history": True,
            "return_images": True,
        }

    def test_instance(self):
        self.assertEqual(self.utilities.partner_id, "001")
        self.assertEqual(self.utilities.api_key, self.public_key)
        self.assertEqual(
            self.utilities.url,
            "https://testapi.smileidentity.com/v1",
        )

    @staticmethod
    def _get_smile_services_response():
        return {
            "id_types": {
                "NG": {
                    "NIN": ["country", "id_type", "id_number", "user_id", "job_id"],
                    "CAC": [
                        "country",
                        "id_type",
                        "id_number",
                        "user_id",
                        "company",
                        "job_id",
                    ],
                    "TIN": ["country", "id_type", "id_number", "user_id", "job_id"],
                    "VOTER_ID": [
                        "country",
                        "id_type",
                        "id_number",
                        "user_id",
                        "job_id",
                    ],
                    "BVN": ["country", "id_type", "id_number", "user_id", "job_id"],
                    "PHONE_NUMBER": [
                        "country",
                        "id_type",
                        "id_number",
                        "user_id",
                        "job_id",
                        "first_name",
                        "last_name",
                    ],
                    "DRIVERS_LICENSE": [
                        "country",
                        "id_type",
                        "id_number",
                        "user_id",
                        "job_id",
                        "first_name",
                        "last_name",
                        "dob",
                    ],
                    "PASSPORT": [
                        "country",
                        "id_type",
                        "id_number",
                        "user_id",
                        "job_id",
                        "first_name",
                        "last_name",
                        "dob",
                    ],
                },
            }
        }

    def test_no_partner_params(self):
        self.__reset_params()
        with self.assertRaises(ValueError) as ve:
            response = Utilities.validate_partner_params(None)
        self.assertEqual(
            ve.exception.args[0], "Please ensure that you send through partner params"
        )

    def test_missing_partner_params(self):
        self.__reset_params()
        self.partner_params["user_id"] = None
        with self.assertRaises(ValueError) as ve:
            response = Utilities.validate_partner_params(self.partner_params)
        value_exception = ve.exception
        self.assertEqual(
            value_exception.args[0],
            "Partner Parameter Arguments may not be null or empty",
        )

        self.__reset_params()
        self.partner_params["job_id"] = None
        with self.assertRaises(ValueError) as ve:
            response = Utilities.validate_partner_params(self.partner_params)
        self.assertEqual(
            ve.exception.args[0],
            "Partner Parameter Arguments may not be null or empty",
        )

        self.__reset_params()
        self.partner_params["job_type"] = None
        with self.assertRaises(ValueError) as ve:
            response = Utilities.validate_partner_params(self.partner_params)
        self.assertEqual(
            ve.exception.args[0],
            "Partner Parameter Arguments may not be null or empty",
        )

    @responses.activate
    def test_validate_id_params_should_raise_when_provided_with_invalid_input(self):
        self.__reset_params()

        self._stub_service("https://testapi.smileidentity.com/v1")
        self.id_info_params["country"] = None
        with self.assertRaises(ValueError) as ve:
            Utilities.validate_id_params(
                self.utilities.url, self.id_info_params, self.partner_params
            )
        self.assertEqual(ve.exception.args[0], "key country cannot be empty")

        self.__reset_params()
        self.id_info_params["id_type"] = None
        with self.assertRaises(ValueError) as ve:
            Utilities.validate_id_params(
                self.utilities.url, self.id_info_params, self.partner_params
            )
        self.assertEqual(ve.exception.args[0], "key id_type cannot be empty")

        self.__reset_params()
        self.id_info_params["id_number"] = None
        with self.assertRaises(ValueError) as ve:
            Utilities.validate_id_params(
                self.utilities.url, self.id_info_params, self.partner_params
            )
        self.assertEqual(ve.exception.args[0], "key id_number cannot be empty")

        self.__reset_params()
        self.id_info_params["country"] = "ZW"
        with self.assertRaises(ValueError) as ve:
            Utilities.validate_id_params(
                self.utilities.url, self.id_info_params, self.partner_params
            )
        self.assertEqual(ve.exception.args[0], "country ZW is invalid")

        self.__reset_params()
        self.id_info_params["id_type"] = "Not_Supported"
        with self.assertRaises(ValueError) as ve:
            Utilities.validate_id_params(
                self.utilities.url, self.id_info_params, self.partner_params
            )
        self.assertEqual(ve.exception.args[0], "id_type Not_Supported is invalid")

        self.__reset_params()
        self.partner_params["user_id"] = None
        with self.assertRaises(ValueError) as ve:
            Utilities.validate_id_params(
                self.utilities.url, self.id_info_params, self.partner_params
            )
        self.assertEqual(ve.exception.args[0], "key user_id cannot be empty")

        self.__reset_params()
        self.id_info_params["first_name"] = None
        with self.assertRaises(ValueError) as ve:
            Utilities.validate_id_params(
                self.utilities.url, self.id_info_params, self.partner_params
            )
        self.assertEqual(ve.exception.args[0], "key first_name cannot be empty")

        self.__reset_params()
        self.partner_params.pop("user_id")
        with self.assertRaises(ValueError) as ve:
            Utilities.validate_id_params(
                self.utilities.url, self.id_info_params, self.partner_params
            )
        self.assertEqual(ve.exception.args[0], "key user_id is required")

    @responses.activate
    def test_get_job_status(self):
        self.__reset_params()
        sec_timestamp = self.signatureObj.generate_sec_key(timestamp=int(time.time()))
        self.stub_get_job_status(sec_timestamp, True)

        job_status = self.utilities.get_job_status(
            self.partner_params, self.options_params, sec_timestamp
        )
        body = {
            "sec_key": sec_timestamp["sec_key"],
            "timestamp": sec_timestamp["timestamp"],
            "partner_id": "001",
            "job_id": self.partner_params["job_id"],
            "user_id": self.partner_params["user_id"],
            "image_links": True,
            "history": True,
        }

        self.assertEqual(job_status.status_code, 200)
        self.assertIsNotNone(job_status.json())
        self.assert_request_called_with(
            "https://testapi.smileidentity.com/v1/job_status", responses.POST, body
        )

    @responses.activate
    def test_get_smile_id_services(self):
        self.__reset_params()

        self._stub_service("https://testapi.smileidentity.com/v1")
        self.utilities.get_smile_id_services(0)

        self._stub_service("https://api.smileidentity.com/v1")
        self.utilities.get_smile_id_services(1)

        self._stub_service("https://random-server.smileidentity.com/v1")
        job_status = self.utilities.get_smile_id_services(
            "https://random-server.smileidentity.com/v1"
        )

        self.assertEqual(job_status.status_code, 200)

    def _stub_service(self, url, json=None):
        if not json:
            json = TestUtilities._get_smile_services_response()

        responses.add(
            responses.GET,
            f"{url}/services",
            json=json,
        )
